@@ -50,16 +50,6 @@ static gboolean         xfmpc_interface_state_event             (XfmpcInterface
                                                                  GdkEventWindowState *event);
 static gboolean         xfmpc_interface_closed                  (XfmpcInterface *interface,
                                                                  GdkEvent *event);
-static void             xfmpc_interface_action_previous         (GtkAction *action,
-                                                                 XfmpcInterface *interface);
-static void             xfmpc_interface_action_pp               (GtkAction *action,
-                                                                 XfmpcInterface *interface);
-static void             xfmpc_interface_action_stop             (GtkAction *action,
-                                                                 XfmpcInterface *interface);
-static void             xfmpc_interface_action_next             (GtkAction *action,
-                                                                 XfmpcInterface *interface);
-static void             xfmpc_interface_action_volume           (GtkAction *action,
-                                                                 XfmpcInterface *interface);
 static void             cb_song_changed                         (XfmpcInterface *interface);
 
 static void             cb_pp_changed                           (XfmpcInterface *interface,
@@ -71,6 +61,18 @@ static void             cb_volume_changed                       (XfmpcInterface
                                                                  gint volume);
 static void             cb_stopped                              (XfmpcInterface *interface);
 
+static void             xfmpc_interface_action_previous         (GtkAction *action,
+                                                                 XfmpcInterface *interface);
+static void             xfmpc_interface_action_pp               (GtkAction *action,
+                                                                 XfmpcInterface *interface);
+static void             xfmpc_interface_action_stop             (GtkAction *action,
+                                                                 XfmpcInterface *interface);
+static void             xfmpc_interface_action_next             (GtkAction *action,
+                                                                 XfmpcInterface *interface);
+static void             xfmpc_interface_action_volume           (GtkAction *action,
+                                                                 XfmpcInterface *interface);
+static void             xfmpc_interface_action_close            (GtkAction *action,
+                                                                 XfmpcInterface *interface);
 
 
 
@@ -109,7 +111,7 @@ static const GtkActionEntry action_entries[] =
   { "stop", NULL, "", "<control>s", NULL, G_CALLBACK (xfmpc_interface_action_stop), },
   { "next", NULL, "", "<control>f", NULL, G_CALLBACK (xfmpc_interface_action_next), },
   { "volume", NULL, "", "<control>v", NULL, G_CALLBACK (xfmpc_interface_action_volume), },
-  { "quit", NULL, "", "<control>q", NULL, G_CALLBACK (gtk_main_quit), },
+  { "quit", NULL, "", "<control>q", NULL, G_CALLBACK (xfmpc_interface_action_close), },
 };
 
 
@@ -210,6 +212,10 @@ xfmpc_interface_init (XfmpcInterface *interface)
 
   control = priv->button_volume = gtk_volume_button_new ();
   gtk_button_set_relief (GTK_BUTTON (control), GTK_RELIEF_NONE);
+  GtkAdjustment* adjustment = gtk_scale_button_get_adjustment (GTK_SCALE_BUTTON (control));
+  adjustment->upper *= 100;
+  adjustment->step_increment *= 100;
+  adjustment->page_increment *= 100;
 
   GtkWidget *progress_box = gtk_event_box_new ();
   control = priv->progress_bar = gtk_progress_bar_new ();
@@ -395,7 +401,7 @@ void
 xfmpc_interface_volume_changed (XfmpcInterface *interface,
                                 gdouble value)
 {
-  xfmpc_mpdclient_set_volume (interface->mpdclient, (guint8)(value * 100));
+  xfmpc_mpdclient_set_volume (interface->mpdclient, value);
 }
 
 void
@@ -404,7 +410,7 @@ xfmpc_interface_set_volume (XfmpcInterface *interface,
 {
   XfmpcInterfacePrivate *priv = XFMPC_INTERFACE_GET_PRIVATE (interface);
 
-  gtk_scale_button_set_value (GTK_SCALE_BUTTON (priv->button_volume), (gdouble)volume / 100);
+  gtk_scale_button_set_value (GTK_SCALE_BUTTON (priv->button_volume), volume);
 }
 
 void
@@ -430,7 +436,8 @@ xfmpc_interface_set_time (XfmpcInterface *interface,
 
   if (G_LIKELY (time_total > 0))
     fraction = (gfloat)time / (gfloat)time_total;
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->progress_bar), fraction);
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->progress_bar),
+                                 (fraction <= 1.0) ? fraction : 1.0);
 }
 
 static gboolean
@@ -608,3 +615,10 @@ xfmpc_interface_action_volume (GtkAction *action,
   g_signal_emit_by_name (priv->button_volume, "popup", G_TYPE_NONE);
 }
 
+static void
+xfmpc_interface_action_close (GtkAction *action,
+                              XfmpcInterface *interface)
+{
+  xfmpc_interface_closed (interface, NULL);
+}
+
