@@ -20,12 +20,38 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <libxfce4util/libxfce4util.h>
 
 #include "interface.h"
 
+
+
+static void
+transform_string_to_int (const GValue *src,
+                         GValue *dst)
+{
+  g_value_set_int (dst, (gint) strtol (g_value_get_string (src), NULL, 10));
+}
+
+static void
+transform_string_to_boolean (const GValue *src,
+                             GValue *dst)
+{
+  g_value_set_boolean (dst, (gboolean) strcmp (g_value_get_string (src), "FALSE") != 0);
+}
+
+
+
 int
 main (int argc, char *argv[])
 {
@@ -33,6 +59,9 @@ main (int argc, char *argv[])
 
   gtk_init (&argc, &argv);
 
+  g_value_register_transform_func (G_TYPE_STRING, G_TYPE_INT, transform_string_to_int);
+  g_value_register_transform_func (G_TYPE_STRING, G_TYPE_BOOLEAN, transform_string_to_boolean);
+
   gtk_window_set_default_icon_name ("xfmpc");
 
   GtkWidget *interface = xfmpc_interface_new ();
