@@ -35,6 +35,7 @@
 
 enum
 {
+  SIG_CONNECTED,
   SIG_SONG_CHANGED,
   SIG_PP_CHANGED,
   SIG_TIME_CHANGED,
@@ -64,6 +65,7 @@ struct _XfmpcMpdclientClass
 {
   GObjectClass              parent_class;
 
+  void (*connected)         (XfmpcMpdclient *mpdclient, gpointer user_data);
   void (*song_changed)      (XfmpcMpdclient *mpdclient, gpointer user_data);
   void (*pp_changed)        (XfmpcMpdclient *mpdclient, gboolean is_playing, gpointer user_data);
   void (*time_changed)      (XfmpcMpdclient *mpdclient, gint time, gint total_time, gpointer user_data);
@@ -132,6 +134,14 @@ xfmpc_mpdclient_class_init (XfmpcMpdclientClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = xfmpc_mpdclient_finalize;
 
+  xfmpc_mpdclient_signals[SIG_CONNECTED] =
+    g_signal_new ("connected", G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (XfmpcMpdclientClass, connected),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   xfmpc_mpdclient_signals[SIG_SONG_CHANGED] =
     g_signal_new ("song-changed", G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST|G_SIGNAL_ACTION,
@@ -263,6 +273,8 @@ xfmpc_mpdclient_connect (XfmpcMpdclient *mpdclient)
 
   mpd_send_password (priv->mi);
 
+  g_signal_emit_by_name (mpdclient, "connected");
+
   return TRUE;
 }
 
@@ -380,6 +392,20 @@ xfmpc_mpdclient_set_song_time (XfmpcMpdclient *mpdclient,
     return TRUE;
 }
 
+
+
+gint
+xfmpc_mpdclient_get_pos (XfmpcMpdclient *mpdclient)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  gint pos = mpd_player_get_current_song_pos (priv->mi);
+  if (pos < 0)
+    pos = 0;
+
+  return pos;
+}
+
 gint
 xfmpc_mpdclient_get_id (XfmpcMpdclient *mpdclient)
 {
@@ -489,6 +515,8 @@ xfmpc_mpdclient_is_stopped (XfmpcMpdclient *mpdclient)
   return mpd_player_get_state (priv->mi) == MPD_PLAYER_STOP;
 }
 
+
+
 void
 xfmpc_mpdclient_update_status (XfmpcMpdclient *mpdclient)
 {
@@ -497,39 +525,6 @@ xfmpc_mpdclient_update_status (XfmpcMpdclient *mpdclient)
   mpd_status_update (priv->mi);
 }
 
-gboolean
-xfmpc_mpdclient_playlist_read (XfmpcMpdclient *mpdclient,
-                               gint *id,
-                               gchar **song,
-                               gchar **length)
-{
-  static MpdData       *data = NULL;
-  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
-
-  if (NULL == data)
-    data = mpd_playlist_get_changes (priv->mi, -1);
-  else
-    data = mpd_data_get_next (data);
-
-  if (NULL != data)
-    {
-      if (NULL != data->song->title)
-        {
-          if (data->song->artist)
-            *song = g_strdup_printf ("%s - %s", data->song->artist, data->song->title);
-          else
-            *song = g_strdup (data->song->title);
-        }
-      else
-        *song = g_path_get_basename (data->song->file);
-
-      *length = g_strdup_printf ("%d:%02d", data->song->time / 60, data->song->time % 60);
-      *id = data->song->id;
-    }
-
-  return NULL != data;
-}
-
 static void
 cb_xfmpc_mpdclient_status_changed (MpdObj *mi,
                                    ChangedStatusType what,
@@ -564,6 +559,8 @@ cb_xfmpc_mpdclient_status_changed (MpdObj *mi,
                            xfmpc_mpdclient_get_total_time (mpdclient));
 }
 
+
+
 gboolean
 xfmpc_mpdclient_queue_commit (XfmpcMpdclient *mpdclient)
 {
@@ -575,6 +572,18 @@ xfmpc_mpdclient_queue_commit (XfmpcMpdclient *mpdclient)
   return TRUE;
 }
 
+gboolean
+xfmpc_mpdclient_queue_add (XfmpcMpdclient *mpdclient,
+                           const gchar *path)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (mpd_playlist_queue_add (priv->mi, (gchar *)path) != MPD_OK)
+    return FALSE;
+
+  return TRUE;
+}
+
 gboolean
 xfmpc_mpdclient_queue_remove_id (XfmpcMpdclient *mpdclient,
                                  gint id)
@@ -587,3 +596,96 @@ xfmpc_mpdclient_queue_remove_id (XfmpcMpdclient *mpdclient,
   return TRUE;
 }
 
+
+
+gboolean
+xfmpc_mpdclient_playlist_read (XfmpcMpdclient *mpdclient,
+                               gint *id,
+                               gchar **song,
+                               gchar **length)
+{
+  static MpdData       *data = NULL;
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (NULL == data)
+    data = mpd_playlist_get_changes (priv->mi, -1);
+  else
+    data = mpd_data_get_next (data);
+
+  if (NULL != data)
+    {
+      if (NULL != data->song->title)
+        {
+          if (data->song->artist)
+            *song = g_strdup_printf ("%s - %s", data->song->artist, data->song->title);
+          else
+            *song = g_strdup (data->song->title);
+        }
+      else
+        *song = g_path_get_basename (data->song->file);
+
+      *length = g_strdup_printf ("%d:%02d", data->song->time / 60, data->song->time % 60);
+      *id = data->song->id;
+    }
+
+  return NULL != data;
+}
+
+gboolean
+xfmpc_mpdclient_database_read (XfmpcMpdclient *mpdclient,
+                               const gchar *dir,
+                               gchar **filename,
+                               gchar **basename,
+                               gboolean *is_dir)
+{
+  static MpdData       *data = NULL;
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT_GET_PRIVATE (mpdclient);
+
+  if (NULL == data)
+    data = mpd_database_get_directory (priv->mi, (gchar *)dir);
+  else
+    data = mpd_data_get_next (data);
+
+  if (NULL != data)
+    {
+      switch (data->type)
+        {
+        case MPD_DATA_TYPE_DIRECTORY:
+          *is_dir = TRUE;
+          *filename = g_strdup (data->directory);
+          *basename = g_path_get_basename (data->directory);
+          break;
+
+        case MPD_DATA_TYPE_SONG:
+          *is_dir = FALSE;
+          *filename = g_strdup (data->song->file);
+
+          if (NULL != data->song->title)
+            {
+              if (data->song->artist)
+                *basename = g_strdup_printf ("%s - %s", data->song->artist, data->song->title);
+              else
+                *basename = g_strdup (data->song->title);
+            }
+          else
+            *basename = g_path_get_basename (data->song->file);
+
+          break;
+
+        case MPD_DATA_TYPE_PLAYLIST:
+#if 0
+          *is_dir = FALSE;
+          *filename = g_strdup (data->playlist);
+          *basename = g_strconcat ("Playlist: ", data->playlist, NULL);
+          break;
+#endif
+
+        default:
+          return xfmpc_mpdclient_database_read (mpdclient, dir, filename, basename, is_dir);
+          break;
+        }
+    }
+
+  return NULL != data;
+}
+
